Module interface

Module interface 

Interface and Interface Methods

Structs

InterfaceInfo
A struct containing metadata for a given interface
MethodInfo
A struct containing metadata for a given method of an interface

Traits

Method
A trait containing the metadata of a given method of an method

Functions

extract_keys
Create an array of Keys from an array of MethodInfos.
flatten
Flattens an array of array of MethodInfos into a flat array of MethodInfos.
resp_body_max_buf_required
Calculated the largest response body payload in bytes.
rqst_body_max_buf_required
Calculated the largest request body payload in bytes.
total_len
Calculates the total number of MethodInfos in an array of array of MethodInfos.