Interface and Interface Methods
Structs
- Interface
Info - A struct containing metadata for a given interface
- Method
Info - A struct containing metadata for a given method of an interface
Traits
- Method
- A trait containing the metadata of a given method of an method
Functions
- extract_
keys - Create an array of
Keys from an array ofMethodInfos. - flatten
- Flattens an array of array of
MethodInfos into a flat array ofMethodInfos. - resp_
body_ max_ buf_ required - Calculated the largest response body payload in bytes.
- rqst_
body_ max_ buf_ required - Calculated the largest request body payload in bytes.
- total_
len - Calculates the total number of
MethodInfos in an array of array ofMethodInfos.