macro_rules! compose_interfaces {
(
mod: $mod_name:ident,
interfaces: [
$(
$($segment:ident)::+$(,)?
)*
]
) => { ... };
}Expand description
Macro to combine multiple interfaces into a single composite interface.
Produces the same module structure as the interface! macro. This does
NOT include the Client trait, it is still necessary to pull individual
Client traits.
Example
ⓘ
compose_interfaces! {
mod: composite,
interfaces: [
basic,
two,
ops,
]
}